@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-
 ------------------------------------------------------------------------------
 -- | A framework for parsing HTTP media type headers.
 module Network.HTTP.Media
@@ -14,6 +12,12 @@ module Network.HTTP.Media
     , (/?)
     , (/.)
 
+    -- * Charsets
+    , Charset
+
+    -- * Encodings
+    , Encoding
+
     -- * Languages
     , Language
     , toParts
@@ -22,6 +26,8 @@ module Network.HTTP.Media
     , matchAccept
     , mapAccept
     , mapAcceptMedia
+    , mapAcceptCharset
+    , mapAcceptEncoding
     , mapAcceptLanguage
     , mapAcceptBytes
 
@@ -29,11 +35,15 @@ module Network.HTTP.Media
     , matchContent
     , mapContent
     , mapContentMedia
+    , mapContentCharset
+    , mapContentEncoding
     , mapContentLanguage
 
     -- * Quality values
     , Quality
     , quality
+    , QualityOrder
+    , qualityOrder
     , maxQuality
     , minQuality
     , parseQuality
@@ -47,20 +57,20 @@ module Network.HTTP.Media
     , RenderHeader (..)
     ) where
 
-#if MIN_VERSION_base(4, 8, 0)
 import           Control.Applicative             ((<|>))
-#else
-import           Control.Applicative             (pure, (<$>), (<*>), (<|>))
-#endif
 
 import qualified Data.ByteString.Char8           as BS
 
 import           Control.Monad                   (guard, (>=>))
 import           Data.ByteString                 (ByteString)
+import           Data.Foldable                   (foldl', maximumBy)
+import           Data.Function                   (on)
 import           Data.Maybe                      (fromMaybe)
 import           Data.Proxy                      (Proxy (Proxy))
 
 import           Network.HTTP.Media.Accept       as Accept
+import           Network.HTTP.Media.Charset      as Charset
+import           Network.HTTP.Media.Encoding     as Encoding
 import           Network.HTTP.Media.Language     as Language
 import           Network.HTTP.Media.MediaType    as MediaType
 import           Network.HTTP.Media.Quality
@@ -122,13 +132,43 @@ mapAcceptMedia ::
 mapAcceptMedia = mapAccept
 
 
+------------------------------------------------------------------------------
+-- | A specialisation of 'mapAccept' that only takes 'Charset' as its input,
+-- to avoid ambiguous-type errors when using string literal overloading.
+--
+-- > getHeader >>= maybe render406Error renderResource . mapAcceptCharset
+-- >     [ ("utf-8",    inUtf8)
+-- >     , ("us-ascii", inAscii)
+-- >     ]
+mapAcceptCharset ::
+    [(Charset, b)]  -- ^ The map of server-side preferences to values
+    -> ByteString   -- ^ The client-side header value
+    -> Maybe b
+mapAcceptCharset = mapAccept
+
+
+------------------------------------------------------------------------------
+-- | A specialisation of 'mapAccept' that only takes 'Encoding' as its input,
+-- to avoid ambiguous-type errors when using string literal overloading.
+--
+-- > getHeader >>= maybe render406Error renderResource . mapAcceptEncoding
+-- >     [ ("compress", compress)
+-- >     , ("identity", id)
+-- >     ]
+mapAcceptEncoding ::
+    [(Encoding, b)]  -- ^ The map of server-side preferences to values
+    -> ByteString    -- ^ The client-side header value
+    -> Maybe b
+mapAcceptEncoding = mapAccept
+
+
 ------------------------------------------------------------------------------
 -- | A specialisation of 'mapAccept' that only takes 'Language' as its input,
 -- to avoid ambiguous-type errors when using string literal overloading.
 --
 -- > getHeader >>= maybe render406Error renderResource . mapAcceptLanguage
--- >     [ ("text/html",        asHtml)
--- >     , ("application/json", asJson)
+-- >     [ ("en-gb", inBritishEnglish)
+-- >     , ("fr",    inFrench)
 -- >     ]
 mapAcceptLanguage ::
     [(Language, b)]  -- ^ The map of server-side preferences to values
@@ -143,8 +183,8 @@ mapAcceptLanguage = mapAccept
 -- overloading.
 --
 -- > getHeader >>= maybe render406Error encodeResourceWith . mapAcceptBytes
--- >     [ ("compress", compress)
--- >     , ("gzip",     gzip)
+-- >     [ ("abc", abc)
+-- >     , ("xyz", xyz)
 -- >     ]
 mapAcceptBytes ::
     [(ByteString, b)]  -- ^ The map of server-side preferences to values
@@ -207,18 +247,50 @@ mapContentMedia
 mapContentMedia = mapContent
 
 
+------------------------------------------------------------------------------
+-- | A specialisation of 'mapContent' that only takes 'Charset' as its input,
+-- to avoid ambiguous-type errors when using string literal overloading.
+--
+-- > getContentCharset >>=
+-- >     maybe send415Error readRequestBodyWith . mapContentCharset
+-- >         [ ("utf-8",    parseUtf8)
+-- >         , ("us-ascii", parseAscii)
+-- >         ]
+mapContentCharset
+    :: [(Charset, b)]  -- ^ The map of server-side responses
+    -> ByteString      -- ^ The client request's header value
+    -> Maybe b
+mapContentCharset = mapContent
+
+
+------------------------------------------------------------------------------
+-- | A specialisation of 'mapContent' that only takes 'Encoding' as its input,
+-- to avoid ambiguous-type errors when using string literal overloading.
+--
+-- > getContentEncoding >>=
+-- >     maybe send415Error readRequestBodyWith . mapContentEncoding
+-- >         [ ("compress", decompress)
+-- >         , ("identity", id)
+-- >         ]
+mapContentEncoding
+    :: [(Encoding, b)]  -- ^ The map of server-side responses
+    -> ByteString       -- ^ The client request's header value
+    -> Maybe b
+mapContentEncoding = mapContent
+
+
 ------------------------------------------------------------------------------
 -- | A specialisation of 'mapContent' that only takes 'Language' as its input,
 -- to avoid ambiguous-type errors when using string literal overloading.
 --
--- > getContentType >>=
+-- > getContentLanguage >>=
 -- >     maybe send415Error readRequestBodyWith . mapContentLanguage
--- >         [ ("application/json", parseJson)
--- >         , ("text/plain",       parseText)
+-- >         [ ("en-gb", parseBritishEnglish)
+-- >         , ("fr",    parseFrench)
 -- >         ]
 mapContentLanguage
     :: [(Language, b)]  -- ^ The map of server-side responses
-    -> ByteString        -- ^ The client request's header value
+    -> ByteString       -- ^ The client request's header value
     -> Maybe b
 mapContentLanguage = mapContent
 
@@ -270,17 +342,17 @@ matchQuality
     -> [Quality a]  -- ^ The pre-parsed client-side header value
     -> Maybe a
 matchQuality options acceptq = do
-    let merge (Quality c q) = map (`Quality` q) $ filter (`matches` c) options
-        matched = concatMap merge acceptq
-        (hq, qs) = foldr qfold (0, []) matched
-        qfold (Quality v q) (mq, vs) = case compare q mq of
-            GT -> (q, [v])
-            EQ -> (mq, v : vs)
-            LT -> (mq, vs)
-        specific (a : ms) = Just $ foldl mostSpecific a ms
-        specific []       = Nothing
-    guard (hq /= 0)
-    specific qs
+    guard $ not (null options)
+    Quality m q <- maximumBy (compare `on` fmap qualityOrder) optionsq
+    guard $ q /= 0
+    return m
+  where
+    optionsq = reverse $ map addQuality options
+    addQuality opt = withQValue opt <$> foldl' (mfold opt) Nothing acceptq
+    withQValue opt qv = qv { qualityData = opt }
+    mfold opt cur acq@(Quality acd _)
+        | opt `matches` acd = mostSpecific acq <$> cur <|> Just acq
+        | otherwise         = cur
 
 
 ------------------------------------------------------------------------------
