@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-
 ------------------------------------------------------------------------------
 -- | Defines the 'Language' accept header with an 'Accept' instance for use in
 -- language negotiation.
@@ -7,17 +5,13 @@ module Network.HTTP.Media.Language.Internal
     ( Language (..)
     ) where
 
-#if !MIN_VERSION_base(4, 8, 0)
-import           Data.Functor                    ((<$>))
-#endif
-
 import qualified Data.ByteString.Char8           as BS
 import qualified Data.CaseInsensitive            as CI
 
 import           Control.Monad                   (guard)
 import           Data.ByteString                 (ByteString)
 import           Data.CaseInsensitive            (CI, original)
-import           Data.Char                       (isAlpha)
+import           Data.Char                       (isAlpha, isAlphaNum)
 import           Data.List                       (isPrefixOf)
 import           Data.Maybe                      (fromMaybe)
 import           Data.String                     (IsString (..))
@@ -28,11 +22,11 @@ import           Network.HTTP.Media.RenderHeader (RenderHeader (..))
 
 ------------------------------------------------------------------------------
 -- | Suitable for HTTP language-ranges as defined in
--- <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4 RFC2616>.
+-- <https://tools.ietf.org/html/rfc4647#section-2.1 RFC4647>.
 --
 -- Specifically:
 --
--- > language-range = ( ( 1*8ALPHA *( "-" 1*8ALPHA ) ) | "*" )
+-- > language-range = (1*8ALPHA *("-" 1*8alphanum)) / "*"
 newtype Language = Language [CI ByteString]
     deriving (Eq, Ord)
 
@@ -55,7 +49,9 @@ instance Accept Language where
       where
         check part = do
             let len = BS.length part
-            guard $ len >= 1 && len <= 8 && BS.all isAlpha part
+            guard $ len >= 1 && len <= 8 &&
+                isAlpha (BS.head part) &&
+                BS.all isAlphaNum (BS.tail part)
             return (CI.mk part)
 
     -- Languages match if the right argument is a prefix of the left.
