@@ -1,21 +1,11 @@
-{-# LANGUAGE CPP #-}
-
 ------------------------------------------------------------------------------
 module Network.HTTP.Media.Accept.Tests (tests) where
 
-------------------------------------------------------------------------------
-#if !MIN_VERSION_base(4, 8, 0)
-import Control.Applicative ((<$>), (<*>))
-#endif
+import           Test.Framework                       (Test, testGroup)
+import           Test.Framework.Providers.QuickCheck2 (testProperty)
 
-------------------------------------------------------------------------------
-import Test.Framework                       (Test, testGroup)
-import Test.Framework.Providers.QuickCheck2 (testProperty)
-import Test.QuickCheck                      ((===))
-
-------------------------------------------------------------------------------
-import Network.HTTP.Media.Accept
-import Network.HTTP.Media.Gen
+import           Network.HTTP.Media.Accept
+import           Network.HTTP.Media.Gen
 
 
 ------------------------------------------------------------------------------
@@ -23,7 +13,6 @@ tests :: [Test]
 tests =
     [ testMatches
     , testMoreSpecificThan
-    , testMostSpecific
     ]
 
 
@@ -46,10 +35,3 @@ testMatches = testGroup "matches"
 testMoreSpecificThan :: Test
 testMoreSpecificThan = testProperty "moreSpecificThan" $
     (not .) . moreSpecificThan <$> genByteString <*> genByteString
-
-
-------------------------------------------------------------------------------
-testMostSpecific :: Test
-testMostSpecific = testProperty "mostSpecific" $ do
-    string <- genByteString
-    (=== string) . mostSpecific string <$> genByteString
