@@ -1,29 +1,24 @@
-{-# LANGUAGE CPP #-}
-
 ------------------------------------------------------------------------------
 module Network.HTTP.Media.Tests (tests) where
 
-------------------------------------------------------------------------------
-#if !MIN_VERSION_base(4, 8, 0)
-import Control.Applicative ((<$>), (<*>))
-#endif
+import           Control.Monad                         (join, replicateM, (>=>))
+import           Data.Foldable                         (foldlM)
+import           Data.Function                         (on)
+import           Data.List                             (nubBy)
+import           Data.Map                              (empty)
+import           Data.Monoid                           ((<>))
+import           Data.Word                             (Word16)
+import           Test.Framework                        (Test, testGroup)
+import           Test.Framework.Providers.QuickCheck2  (testProperty)
+import           Test.QuickCheck
 
-------------------------------------------------------------------------------
-import Control.Monad                        (join, replicateM, (>=>))
-import Data.Foldable                        (foldlM)
-import Data.Map                             (empty)
-import Data.Monoid                          ((<>))
-import Data.Word                            (Word16)
-import Test.Framework                       (Test, testGroup)
-import Test.Framework.Providers.QuickCheck2 (testProperty)
-import Test.QuickCheck
+import           Network.HTTP.Media                    hiding (parameters,
+                                                        subType)
+import           Network.HTTP.Media.Gen                (padString)
+import           Network.HTTP.Media.MediaType.Gen
+import           Network.HTTP.Media.MediaType.Internal
+import           Network.HTTP.Media.Quality
 
-------------------------------------------------------------------------------
-import Network.HTTP.Media                    hiding (parameters, subType)
-import Network.HTTP.Media.Gen                (padString)
-import Network.HTTP.Media.MediaType.Gen
-import Network.HTTP.Media.MediaType.Internal
-import Network.HTTP.Media.Quality
 
 ------------------------------------------------------------------------------
 tests :: [Test]
@@ -129,14 +124,7 @@ testMatch
     -> ([Quality MediaType] -> a)
     -> Test
 testMatch name match qToI = testGroup ("match" ++ name)
-    [ testProperty "Highest quality" $ do
-        server <- genServer
-        qs     <- replicateM (length server) $ choose (1, 1000)
-        let client = zipWith Quality server qs
-            qmax v q = if qualityValue q > qualityValue v then q else v
-        return $ match server (qToI client) ===
-            Just (qualityData $ foldr1 qmax client)
-    , testProperty "Most specific" $ do
+    [ testProperty "Most specific" $ do
         media <- genConcreteMediaType
         let client = qToI $ map maxQuality
                 [ MediaType "*" "*" empty
@@ -149,22 +137,73 @@ testMatch name match qToI = testGroup ("match" ++ name)
         client <- listOf1 genConcreteMediaType
         server <- filter (not . flip any client . matches) <$> genServer
         return $ match server (qToI $ map maxQuality client) === Nothing
-    , testProperty "Never chooses q=0" $ do
-        server <- genServer
-        return $ match server (qToI $ map minQuality server) === Nothing
     , testProperty "Left biased" $ do
-        server <- genServer
+        server <- genNubServer
         let client = qToI $ map maxQuality server
         return $ match server client === Just (head server)
     , testProperty "Against */*" $ do
-        server <- genServer
+        server <- genNubServer
         let stars = "*/*" :: MediaType
         return $ match server (qToI [maxQuality stars]) ===
             Just (head server)
     , testProperty "Against type/*" $ do
-        server <- genServer
+        server <- genNubServer
         let client = qToI [maxQuality (subStarOf $ head server)]
         return $ match server client === Just (head server)
+    , testQuality match qToI
+    ]
+
+
+------------------------------------------------------------------------------
+testQuality
+    :: ([MediaType] -> a -> Maybe MediaType)
+    -> ([Quality MediaType] -> a)
+    -> Test
+testQuality match qToI = testGroup "Quality"
+    [ testProperty "Highest quality" $ do
+        server <- genServer
+        qs     <- replicateM (length server) $ choose (1, 1000)
+        let client = zipWith Quality server qs
+            qmax v q = if qualityValue q > qualityValue v then q else v
+        return $ match server (qToI client) ===
+            Just (qualityData $ foldr1 qmax client)
+    , testProperty "Most specific quality" $ do
+        (a, b) <- genMatchingPair
+        c      <- genDiffMediaType a
+        let client = qToI [quality a "0.5", maxQuality b, maxQuality c]
+        return $ match [a, c] client === Just c
+    , testQ0 match qToI
+    ]
+
+
+------------------------------------------------------------------------------
+testQ0
+    :: ([MediaType] -> a -> Maybe MediaType)
+    -> ([Quality MediaType] -> a)
+    -> Test
+testQ0 match qToI = testGroup "q=0"
+    [ testProperty "Does not choose a q=0" $ do
+        server <- genConcreteMediaType
+        return $ match [server] (qToI [minQuality server]) === Nothing
+    , testProperty "Does not choose any q=0" $ do
+        server <- genServer
+        return $ match server (qToI $ map minQuality server) === Nothing
+    , testProperty "Does not choose q=0 with less specific type" $ do
+        (a, b) <- genMatchingPair
+        let client = qToI [minQuality a, maxQuality b]
+        return $ match [a] client === Nothing
+    , testProperty "Does choose type with q=0 on less specific type" $ do
+        (a, b) <- genMatchingPair
+        let client = qToI [minQuality b, maxQuality a]
+        return $ match [a] client === Just a
+    , testProperty "Does not choose q=0 when followed by same type" $ do
+        server <- genConcreteMediaType
+        let client = qToI [minQuality server, maxQuality server]
+        return $ match [server] client === Nothing
+    , testProperty "Does not choose q=0 when preceded by same type" $ do
+        server <- genConcreteMediaType
+        let client = qToI [maxQuality server, minQuality server]
+        return $ match [server] client === Nothing
     ]
 
 
@@ -195,6 +234,11 @@ genServer :: Gen [MediaType]
 genServer = listOf1 genConcreteMediaType
 
 
+------------------------------------------------------------------------------
+genNubServer :: Gen [MediaType]
+genNubServer = nubBy (on (==) stripParams) <$> genServer
+
+
 ------------------------------------------------------------------------------
 genServerAndClient :: Gen ([MediaType], [MediaType])
 genServerAndClient = do
